/* ************************************************************************* *
   xmem - (C) Copyright 1996-1997 Robert Gasch (Robert_Gasch@peoplesoft.com)
	  http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */



#include "xmem.h"

extern Display			*mydisplay;
extern char*			myname;			/* Name we hunt resources with */
extern XrmDatabase		resourceDB;		/* X resources. */
extern WinType			main_win,		/* main window */
				menu[],			/* menu items */
				mem_win[],		/* FS graphs */
				global_win;		/* compatability in util.c */
extern XGCValues		gray_gc_val;		/* GC values for gray fill */
extern XEvent			myevent;		/* event */
extern unsigned long		fg, bg;			/* forgrund, background */
extern Pixmap			warn_pix;		/* Warning Pixmap */
extern int 			upd_interval,		/* when do we update stats */
				NMW,			/* number of file systems */
				detail_open,		/* which mem_wind is open */
				detail_share_color, 	/* */
				minimize,		/* minimize dimensions */
				menu_border,		/* menu border */
				fs_border,		/* file system border */
				percent,		/* show percent */
				absolute,		/* make bar size relative */
				verbose,		/* verbose mode */
				sort,			/* do we sort by name? */
				gray,			/* use gray fill */
				available,		/* do we show what's available */
				show_menu;		/* do we show the menu? */

extern unsigned int 		main_mem, 
				used_mem, 
				free_mem, 
				shared_mem, 
				buf_mem,
				swap_mem, 
				used_swap, 
				free_swap,
				mem[], 
				biggest_mem;

extern unsigned long		turns;			/* how many turns */
extern float			mem_perc[],		/* percentage use for memory */
				warn_val;

extern MyString			hname,
				MemType[];


/* *** variables accessed only in util.c - make them static to make sure *** */
static char 	*Menu_Text[MENU_ITEMS]={
			" Update Now ",
			"    Quit    "};

static int	percent_width;




/* ************************************************************************ */
/* ************************ set the menu's position *********************** */
/* ************************************************************************ */
void fix_menu_pos (width)
int width;
{
	int i;

	if (show_menu)
		for (i=0; i<MENU_ITEMS; i++) 
			{
			menu[i].x = ((width - SM_MENU_WIDTH(menu[i])) / 2);
			if (turns)
				XMoveWindow (mydisplay,menu[i].win,menu[i].x,menu[i].y);
			}
}



/* ************************************************************************ */
/* ******************* this sets the window's dimensions ****************** */
/* ************************************************************************ */
void set_mem_win_size (i, ppnfs)
int i, ppnfs;
{
	mem_win[i].x = OFF_X;
	mem_win[i].width = main_win.width - OFF_X * 2;
	mem_win[i].height = ppnfs - LETTER_SPACE(mem_win[i]);
	if (show_menu)
		mem_win[i].y = (BEGIN_NMW(mem_win[i]) + (ppnfs * i));
	else
		mem_win[i].y = (LETTER_SPACE(mem_win[i])*(i+1))+mem_win[i].height*i;

	if (turns)
		fix_mem_win_width (i);

#ifdef DEBUG
  printf ("%d %d %d %d\n", mem_win[i].x, mem_win[i].y, mem_win[i].width,
	  mem_win[i].height);
#endif
}



/* *********************************************************************** */
/* ********* toggle the display mode and update the windows ************** */
/* *********************************************************************** */
void toggle_mode ()
{
	int i;

	if (absolute)
		absolute=FALSE;
	else
		absolute=TRUE;
	for (i=0; i<MAXMEMZONES; i++)
		{
		mem_win[i].width=main_win.width-OFF_X*2;
		if (!absolute)
			fix_mem_win_width (i);
		XResizeWindow (mydisplay,mem_win[i].win, mem_win[i].width,
			mem_win[i].height);
		redraw_mem_win (i);
		}

}



/* *********************************************************************** */
/* *********************** redraw the main window ************************ */
/* *********************************************************************** */
void redraw_main_win ()
{
	int 		i, 
			text_width = main_win.width-(OFF_X+OFF_X+
				percent_width*(percent|available));

	for (i = 0; i < MAXMEMZONES; i++) 
		{
		int n = strlen(MemType[i]);

		/* *** find how much of the name we can write *** */
		while (XTextWidth(main_win.font_info, MemType[i], n) > 
			text_width)
				n--;
		XDrawImageString (mydisplay, main_win.win, 
			main_win.gc, OFF_X + 2,
			mem_win[i].y - 2, MemType[i], n);

		/* *** handle the exclamation mark after the FS name *** */
		if (warn_val > 0) 
			{
			if (mem_perc[i] >= warn_val)
				XCopyArea (mydisplay, warn_pix, main_win.win, 
				main_win.gc, 0, 0, warn_width, 
				warn_height, (main_win.width - OFF_X) + 2,
		   		mem_win[i].y - LETTER_HEIGHT(main_win));
			else
				XClearArea (mydisplay, main_win.win, 
					(main_win.width - OFF_X) + 2, 
					mem_win[i].y - LETTER_HEIGHT(main_win), 
					warn_width, warn_height, FALSE);
			}
		/* ** handle the color warning levels *** */
		if (warn_val > -1)
			XSetForeground (mydisplay, mem_win[i].gc, mem_win[i].fg);

		if (percent)
			write_percent (i);
		}
} 



/* *********************************************************************** */
/* *************************** redraw the bars *************************** */
/* *********************************************************************** */
void redraw_mem_win(i)
int i;
{
	/* *** now draw bar *** */
	/* XClearWindow (mydisplay, mem_win[i].win); */
	XFillRectangle (mydisplay, mem_win[i].win, mem_win[i].gc, 0, 0,
		(int)(mem_perc[i]*mem_win[i].width), mem_win[i].height);
	XClearArea(mydisplay, mem_win[i].win,
		(int)(mem_perc[i] * mem_win[i].width), 0, 0, 0, False);
}




/* *********************************************************************** */
/* ************************* write the percentage ************************ */
/* *********************************************************************** */
void write_percent(i)
int i;
{
	char 	s[30];

	sprintf (s, "%3d%%", (int)(mem_perc[i] * 100));
	XDrawImageString (mydisplay, main_win.win, main_win.gc, 
		(main_win.width - OFF_Y -
		XTextWidth(main_win.font_info, s, strlen(s))),
 		mem_win[i].y - 2, s, strlen (s));
}



/* *********************************************************************** */
/* ************************* open a detail window ************************ */
/* *********************************************************************** */
void open_detail_win (argc, argv)
int	argc;
char 	**argv;
{
	XSizeHints szhint;
	XClassHint clshint;
	XWMHints wmhints;
	  
	szhint.min_width  = DETAIL_X;
	szhint.min_height = DETAIL_Y;
	szhint.max_width  = szhint.min_width;
	szhint.max_height = szhint.min_height;
	szhint.x = global_win.x;
	szhint.y = global_win.y;
	szhint.width = global_win.width;
	szhint.height = global_win.height;
	szhint.flags = global_win.flags | PMinSize | PMaxSize | PSize;

	clshint.res_name  = "detailWin";
	clshint.res_class = CLASS_NAME;

	wmhints.input = True;
	wmhints.initial_state = NormalState;
	wmhints.window_group = main_win.win;
	wmhints.flags = InputHint | StateHint | WindowGroupHint;

	create_window(DefaultRootWindow(mydisplay), &global_win,
		getColorResource(resourceDB,
		catlist(myname, ".detailWin.foreground",
		(char*) NULL), "Xofm.DetailWin.Foreground", fg),
		getColorResource(resourceDB, catlist(myname,
			".detailWin.background", (char*) NULL),
			"Xofm.DetailWin.Background", bg));

	/* *** set up the properties for the window. *** */
	XSetStandardProperties (mydisplay, global_win.win, 
		global_win.text, global_win.text, None, argv, argc, 
		&szhint);
	XSetClassHint(mydisplay, global_win.win, &clshint);
	XSetWMHints(mydisplay, global_win.win, &wmhints);

	XMapRaised (mydisplay, global_win.win);
}



/* *********************************************************************** */
/* *********************** create a single window ************************ */
/* *********************************************************************** */
void create_window (parent_win, this_win, foreg, backg)
Window	parent_win; 
WinTypePtr this_win;
unsigned long foreg, backg;
{
	/* *** Create Actual Window *** */
	this_win->win = XCreateSimpleWindow (mydisplay, parent_win, this_win->x,
		this_win->y, this_win->width, this_win->height, 
		this_win->line_thick, foreg, backg);

	if (!this_win->win) 
		{
		fprintf (stderr,
			"Error creating Simple Window %s ... Exiting.\n",
			this_win->text);
		exit (1);
		}

#ifdef DEBUG
	fprintf (stdout, "Created Simple Window %s (%d) %d %d %d %d\n", 
		this_win->text, this_win->win, this_win->x, this_win->y,
		this_win->width, this_win->height, this_win->line_thick);
#endif

	/* *** set Graphics Content creation and initialize *** */
	this_win->gc = XCreateGC (mydisplay, this_win->win, 0, 0);

	if (! this_win->gc) 
		{
		fprintf (stderr, "Error creating GC for  %s ... Exiting.\n",
			this_win->text);
		exit (1);
		}

#ifdef DEBUG
	fprintf (stdout, "Created GC %d\n", this_win->gc);
#endif

	XSetBackground (mydisplay, this_win->gc, backg);
	XSetForeground (mydisplay, this_win->gc, foreg);

	if (this_win->font_info != 0)
		XSetFont(mydisplay, this_win->gc, this_win->font_info->fid);

	this_win->fg = foreg;
	this_win->bg = backg;

	/* *** specify which input we want this window to process *** */
	XSelectInput (mydisplay, this_win->win, this_win->event_mask);
}




/* *********************************************************************** */
/* highlight menu item (window) in response to a mouse entering or leaving */
/* *********************************************************************** */
int highlight_menu (menu, menu_num,highlight)
WinType menu[];
int	menu_num;
int	highlight;
{
	int i, state=NOGOOD;

	if (!show_menu)
		return (NOGOOD);

	/* *** figure out which menu *** */
	for (i = 0; i < menu_num; i++) 
		if (myevent.xcrossing.window == menu[i].win) 
			{
			state = i;
			i = menu_num;
			}

	/* *** check if menu is valid and change fg & bg *** */
	if (state >= 0 && state < menu_num) 
		{
		if (highlight) 
			{
#ifdef DEBUG
			fprintf(stdout, "Enter state in button %s\n", 
				menu[state].text);
#endif
		XSetBackground (mydisplay, menu[state].gc, fg);
		XSetForeground (mydisplay, menu[state].gc, bg);
			} 
		else 
			{
#ifdef DEBUG
			fprintf (stdout, "Exit state in button %s\n", 
				menu[state].text);
#endif
			XSetBackground (mydisplay, menu[state].gc, bg);
			XSetForeground (mydisplay, menu[state].gc, fg);
			}

		for (i = 0; i < menu_num; ++i)
			XDrawImageString (mydisplay, menu[i].win, menu[i].gc, 1,
			LETTER_ASCENT(menu[i]), menu[i].text, 
			strlen(menu[i].text)); 
		return (state);
	} 
	else
		return (NOGOOD);
}


/* *********************************************************************** */
/* ************** redraw the string in an exposed window ***************** */
/* *********************************************************************** */
int expose_win (menu, menu_num)
WinType	menu[];
int	menu_num;
{
	int i;

	if (!show_menu)
		return (TRUE);

	for (i = 0; i < menu_num; i++) 
		{
		if (myevent.xexpose.window == menu[i].win) 
			{
			XDrawImageString (mydisplay, menu[i].win, menu[i].gc, 1,
				LETTER_ASCENT(menu[i]), menu[i].text,
				strlen(menu[i].text));
			return (FALSE);
			}
		}

	return (TRUE);
}




/* *********************************************************************** */
/* *********************** destroy an entire menu  *********************** */
/* *********************************************************************** */
void destroy_menu (menu, menu_num)
WinType	menu[];
int	menu_num;
{
	int i;

	for (i=0; i<menu_num; i++)
		{
		XDestroyWindow (mydisplay, menu[i].win);
#ifdef DEBUG
		fprintf (stdout, "Destroyed menu: %s\n", menu[i].text);
#endif
		XFreeGC (mydisplay, menu[i].gc);
#ifdef DEBUG
		fprintf (stdout, "Destroyed GC: %d\n", menu[i].gc);
#endif
		}
}




/* *********************************************************************** */
/* *************** return the number of the button pressed *************** */
/* *********************************************************************** */
int which_button_pressed (menu, menu_num)
WinType	menu[];
int	menu_num;
{
	int i;

	for (i=0; i<menu_num; i++)
		if (myevent.xbutton.window == menu[i].win)
			return (i);
	return (NOGOOD);
}



/* ***************************************************************** */
/* ********** this basically inits the window definitions ********** */
/* ***************************************************************** */
void init_all_windows ()
{
	int i, x, y, flags, geom_used=FALSE, ppnfs;
	unsigned long small_event_mask, big_event_mask, fs_mask;
	unsigned int width, height;
	unsigned int border_width =
		getIntResource(resourceDB, catlist(myname, ".borderWidth", 
			(char*) NULL), "Xmem.BorderWidth", DEF_BORDER_WIDTH);
	XFontStruct* def_font = XLoadQueryFont(mydisplay, DEFAULT_FONT);
	XrmString str_type;
	XrmValue value;
	small_event_mask = ButtonPressMask | StructureNotifyMask | 
		ExposureMask| KeyPressMask | EnterWindowMask | LeaveWindowMask;
	big_event_mask = ButtonPressMask | ExposureMask | EnterWindowMask |
		LeaveWindowMask;
	fs_mask = ButtonPressMask | ExposureMask;

	if (def_font == NULL) 
		{
		fprintf(stderr, "Could not load default font \"%s\".\n", 
			DEFAULT_FONT);
		exit(1);
		}

	sprintf (main_win.text, "xmem %s", hname);

	main_win.x = WIN_X;
	main_win.y = WIN_Y;
	main_win.font_info = getFontResource (resourceDB, 
		catlist(myname, ".mainWin.font", (char*) NULL),
		"Xmem.MainWin.Font", def_font);
	if (minimize)	
		{
			main_win.width = MIN_WIN_X;
			if (show_menu)
				main_win.height = ((MIN_INTERVAL(main_win) * 
					NMW) + BEGIN_NMW(main_win));
			else
				main_win.height = ((MIN_INTERVAL(main_win) * 
					NMW) + MENU_HEIGHT(main_win));
		} 
	else 
		{
		main_win.width = WIN_X;
		if (show_menu)
			main_win.height = ((INTERVAL(main_win) * NMW) + 
				BEGIN_NMW(main_win));
		else
			main_win.height = ((INTERVAL(main_win) * 
				NMW) + MENU_HEIGHT(main_win)); 
		}
	main_win.line_thick = border_width;
	main_win.event_mask = small_event_mask;
	main_win.flags = PPosition;

	if (XrmGetResource(resourceDB, catlist(myname, ".mainWin.geometry", 
		(char*) NULL), "Xmem.MainWin.Geometry", &str_type, &value)) 
		{
		flags = XParseGeometry(value.addr, &x, &y, &width, &height);
		if (XValue & flags) 
			{
			if (XNegative & flags)
			   x = DisplayWidth(mydisplay, 
				DefaultScreen(mydisplay)) + x -
				main_win.width - 2 * main_win.line_thick;
			main_win.x = x;
			main_win.flags = USPosition;
			}

		if (YValue & flags) 
			{
			if (YNegative & flags)
			   y = DisplayHeight(mydisplay, 
				DefaultScreen(mydisplay)) + y -
				main_win.height - 2 * main_win.line_thick;
			main_win.y = y;
			main_win.flags = USPosition;
			}
		if (WidthValue & flags)
			{
			if (width < MIN_WIN_X)
				{
				width = MIN_WIN_X;
				if (verbose)
					fprintf (stderr, "Specified width too \
small ... adjusting to minimum size\n");
				}
			else
			if (width > 
			    DisplayWidth(mydisplay, DefaultScreen(mydisplay)))
				{
				width = DisplayWidth(mydisplay, 
					DefaultScreen(mydisplay))-1;
				if (verbose)
					fprintf (stderr, "Specified width too \
large ... adjusting to maximum size\n");
				}
			main_win.width = width;
			main_win.flags = USPosition;
			geom_used=TRUE;
			}
		if (HeightValue & flags)
			{
			if (height < 
			    ((MIN_INTERVAL(main_win)*NMW)+BEGIN_NMW(main_win)))
				{
				height = ((MIN_INTERVAL(main_win)*NMW)+
					BEGIN_NMW(main_win));
				if (verbose)
					fprintf (stderr, "Specified height too \
small ... adjusting to minimum size\n");
				}
			else
			if (height > 
			    DisplayHeight(mydisplay, DefaultScreen(mydisplay)))
				{
				height = DisplayHeight(mydisplay,
					DefaultScreen(mydisplay))-1;
				if (verbose)
					fprintf (stderr, "Specified height too \
large ... adjusting to maximum size\n");
				}
			main_win.height = height;
			main_win.flags = USPosition;
			geom_used=TRUE;
			}
		}

	for (i = 0; i < MENU_ITEMS; i++) 
		{
		strcpy(menu[i].text, Menu_Text[i]);
		menu[i].font_info = main_win.font_info;
		menu[i].width = SM_MENU_WIDTH(menu[i]);
		menu[i].height = MENU_HEIGHT(menu[i]);
		menu[i].x = ((main_win.width - menu[i].width) / 2);
		menu[i].y = MENU_Y + MENU_HEIGHT(menu[i]) * i;
		if (menu_border)
			menu[i].line_thick = 1;
		else
			menu[i].line_thick = 0;
		menu[i].event_mask = big_event_mask;
		}

	ppnfs = ((main_win.height-BEGIN_NMW(main_win)) / NMW);
	for (i = 0; i < MAXMEMZONES; i++) 
		{
		mem_win[i].font_info = main_win.font_info;
		mem_win[i].line_thick = fs_border;
		mem_win[i].event_mask = fs_mask;
		}

	for (i = 0; i < MAXMEMZONES; i++) 
		{
		mem_win[i].x = OFF_X;
		if (geom_used)
			set_mem_win_size (i, ppnfs);
		else
		if (minimize) 
			{
			if (show_menu)
				{
				mem_win[i].y = ((MIN_INTERVAL(mem_win[i]) * i) + 
					BEGIN_NMW(mem_win[i]));
				mem_win[i].height = MIN_MENU_HEIGHT;
				}
			else
				{
				mem_win[i].y = MIN_INTERVAL(mem_win[i])*i+MENU_HEIGHT(mem_win[i])*1.5;
				mem_win[i].height = MIN_MENU_HEIGHT;
				}
			} 
		else 
			{
			if (show_menu)
				{
				mem_win[i].y = ((INTERVAL(mem_win[i]) * i) + 
					BEGIN_NMW(mem_win[i]));
				mem_win[i].height = MENU_HEIGHT(mem_win[i]);
				}
			else
				{
				mem_win[i].y = (INTERVAL(mem_win[i])*i)+MENU_HEIGHT(mem_win[i])*1.5;
				mem_win[i].height = MENU_HEIGHT(mem_win[i]);
				}
			}
		mem_win[i].width = main_win.width - OFF_X * 2; 
		}
	sprintf (global_win.text, "%s detail", hname);
	global_win.x = OFF_X;
	global_win.y = OFF_Y;
	global_win.font_info = getFontResource(resourceDB, 
		catlist(myname, ".detailWin.font", (char*) NULL),
		"Xmem.DetailWin.Font", def_font);
	global_win.width = DETAIL_X;
	global_win.height = DETAIL_Y;
	global_win.line_thick = border_width;
	global_win.event_mask = small_event_mask;
	global_win.flags = PPosition;
	if (XrmGetResource(resourceDB,
		catlist(myname, ".detailWin.geometry", (char*) NULL),
		"Xmem.DetailWin.Geometry", &str_type, &value)) 
		{
		flags = XParseGeometry(value.addr, &x, &y, &width, &height);
		if (XValue & flags) 
			{
 			if (XNegative & flags)
			   x  = DisplayWidth(mydisplay, 
				DefaultScreen(mydisplay)) + x -
				global_win.width - 2 * global_win.line_thick;
			global_win.x = x;
			global_win.flags = USPosition;
			}
		if (YValue & flags) 
			{
			if (YNegative & flags)
			   y = DisplayHeight(mydisplay, 
				DefaultScreen(mydisplay)) + y -
				global_win.height - 2 * global_win.line_thick;
			global_win.y = y;
			global_win.flags = USPosition;
			}
		}

	/* *** 4 spaces (3 digits + 1 %) + 1 space *** */
	percent_width = XTextWidth(main_win.font_info, "9999%", 5);
}



/* ***************************************************************** */
/* ********** Returns the value of the given bool resource ********** */
/* ***************************************************************** */
Bool getBoolResource(db, str_name, str_class, deflt)
XrmDatabase db;
char* str_name;
char* str_class;
Bool deflt;
{
	XrmString str_type;
	XrmValue value;

	if (XrmGetResource(db, str_name, str_class, &str_type, &value)) 
		{
		return ((!strcmp(value.addr, "true")) || 
			(!strcmp(value.addr, "True")) ||
			(!strcmp(value.addr, "TRUE")) ||
			(!strcmp(value.addr, "yes")) ||
			(!strcmp(value.addr, "Yes")) ||
			(!strcmp(value.addr, "YES")) ||
			(!strcmp(value.addr, "on")) || 
			(!strcmp(value.addr, "On")) || 
			(!strcmp(value.addr, "ON")) || 
			(!strcmp(value.addr, "1")));
		} 
	else
		return (deflt);
}



/* ***************************************************************** */
/* ********** Returns the value of the given int resource ********** */
/* ***************************************************************** */
int getIntResource(db, str_name, str_class, deflt)
XrmDatabase db;
char* str_name;
char* str_class;
int deflt;
{
	XrmString str_type;
	XrmValue value;

	if (XrmGetResource(db, str_name, str_class, &str_type, &value))
		return (atoi(value.addr));
	else
		return (deflt);
}



/* ***************************************************************** */
/* ********** Returns the value of the given Font resource ********* */
/* ***************************************************************** */
XFontStruct* getFontResource(db, str_name, str_class, deflt)
XrmDatabase db;
char* str_name;
char* str_class;
XFontStruct* deflt;
{
	XrmString str_type;
	XrmValue value;

	if (XrmGetResource(db, str_name, str_class, &str_type, &value)) 
		{
		XFontStruct* font = XLoadQueryFont(mydisplay, value.addr);
		if (font == NULL) 
			{
			fprintf(stderr, 
				"Could not load font \"%s\".\n", value.addr);
			return (deflt);
			} 
		else
			return (font);
  		} 
	else
		return (deflt);
}



/* ***************************************************************** */
/* ********** Returns the value of the given color resource ******** */
/* ***************************************************************** */
unsigned long getColorResource(db, str_name, str_class, deflt)
XrmDatabase db;
char* str_name;
char* str_class;
unsigned long deflt;
{
	XrmString str_type;
	XrmValue value;

	if (XrmGetResource(db, str_name, str_class, &str_type, &value)) 
		{
		XColor screen_def;
		if (XParseColor(mydisplay, DefaultColormapOfScreen
			(DefaultScreenOfDisplay(mydisplay)), value.addr, 
			&screen_def) == 0 
		    ||
		    XAllocColor(mydisplay, DefaultColormapOfScreen
			(DefaultScreenOfDisplay(mydisplay)), &screen_def) == 0)
			{
			fprintf(stderr, 
			   "Color specification \"%s\" invalid.\n", value.addr);
			return (deflt);
			} 
		else
 			return (screen_def.pixel);
		} 
	else
		return (deflt);
}



/* ***************************************************************** */
/* ********** Returns the concatenation of the NULL       ********** */
/* ********** terminated list of strings.		  ********** */
/* ***************************************************************** */
#ifdef __STDC__
#if NeedFunctionPrototypes
char* catlist(char *str1, ...)
#else
char* catlist(str1)
char* str1;
#endif /* NeedFunctionPrototypes */
{
	static char blocks[2][64];
	static int current = 0;
	va_list ap;
	char* c;

	strcpy(blocks[current], str1);
	va_start(ap, str1);
	while ((c = va_arg(ap, char*)) != NULL)
		strcat(blocks[current], c);
	va_end(ap);
	c = blocks[current];
	current = (current + 1) % 2;
	return c;
}
#else
char* catlist(va_alist)
va_dcl
{
	static char blocks[2][64];
	static int current = 0;
	va_list ap;
	char* c;

	va_start(ap);
	blocks[current][0] = '\0';
	while ((c = va_arg(ap, char*)) != NULL)
		strcat(blocks[current], c);
	va_end(ap);
	c = blocks[current];
	current = (current + 1) % 2;
	return c;
}
#endif /* __STDC__ */


