/* ************************************************************************* *
   xmem - (C) Copyright 1996-1997 Robert Gasch (Robert_Gasch@peoplesoft.com)
	  http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */



/* *************************************************************************
   This program displays a list of bar graphs for the file systems of the 
   machine you are on. The following switches are supported:

	+rv			reverse video (use to override xrdb entry) 
	+synchronous		syncronous mode (use to override xrdb entry)
	-?			help
	-a			absolute display mode
	-background <arg>	backgound color 
	-b			black bars (disable gray fill)
	-bg <arg>		same as -background 
	-bordercolor <arg>	border color
	-borderwidth <arg>	border width
	-detailgeometry <arg> 	Geometry of detail window. Only X and Y are 
					honoured
	-display		display
	-fg <arg>		same as -forground
	-font <arg>		font
	-foreground <arg>	forground color (also file system bar color)
	-geometry <arg>		geometry (will override extreme window sizes to 
					apply reasonable settings)
	-help			help
	-i <arg>		interval at which stats are updated(default=60s)
	-iconic			iconic
	-m			minimize window size
	-mb			draw menu borders
	-menu 			show the menu
	-name <arg>		run xmem under this name
	-p			don't display percentages
	-rv 			reverse video
	-synchronous		synchronous mode
	-v			verbose
	-w <arg>		display warning pixmap when (real) memory usage 
					reaches <arg> %
	-xrm                    set an entry in the resource database for this
					execution only
	help			help



   Version History:
	30-04-96: 1.00 
			Started adapting xfsm code to make xmem. 
	01-09-96: 1.01
			Adapted code to work under new (2.0) kernels. 
	10-08-97: 1.02
			Fixed incorrect swap detail display

  ************************************************************************* */


#include "xmem.h"

Display			*mydisplay;
char*			myname;		/* Name we hunt resources with */
XrmDatabase		resourceDB;		/* X resources. */
WinType			main_win,		/* main window */
			menu[MENU_ITEMS],	/* menu items */
			mem_win[MAXMEMZONES],	/* FS graphs */
			global_win;		/* compatability in util.c */
XGCValues		gray_gc_val;		/* GC values for gray fill */
XEvent			myevent;		/* event */
XSizeHints		mainwin_hint;		/* hints for main win */
unsigned long		fg, bg;			/* forgrund, background */
Pixmap			warn_pix;		/* Warning Pixmap */
int 			upd_interval=DEF_UPD_INT,/* when do we update stats */
			NMW=MAXMEMZONES,	/* number of file systems */
			detail_open=NOGOOD,	/* which mem_wind is open */
			detail_share_color = FALSE, /* */
			minimize=FALSE,		/* minimize dimensions */
			menu_border=FALSE,	/* menu border */
			fs_border=1,		/* file system border */
			percent=TRUE,		/* show percent */
			absolute=FALSE,		/* make bar size relative */
			verbose=FALSE,		/* verbose mode */
			sort=FALSE,		/* do we sort by name? */
			gray=TRUE,		/* use gray fill */
			available=FALSE,	/* do we show what's available */
			show_menu=FALSE;	/* do we show the menu? */

unsigned int 		main_mem, 
			used_mem, 
			free_mem, 
			shared_mem, 
			buf_mem,
			cached_mem,
			swap_mem, 
			used_swap, 
			free_swap,
			mem[MAXMEMZONES],
			biggest_mem;

unsigned long		turns=0;		/* how many turns */
float			mem_perc[MAXMEMZONES],	/* percentage use for memory */
			warn_val=0;
MyString		hname,			/* hostname */
			MemType [MAXMEMZONES]={
				"Real",
				"Virtual"};


/* *** get the gray bitmap and define our warning bitmap *** */
#include <X11/bitmaps/gray1>
static char warn_bits[] = {
   0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x00, 0x06, 0x06};

static XrmOptionDescRec opTable[] = {
		/* KEEP THIS SORTED BY THE FIRST ARGUMENT! */
/* the char * should be XPointer but many systems don't support it */
{"+rv",		"*reverseVideo", XrmoptionNoArg,	(char *) "off"},
{"+synchronous","*synchronous",	XrmoptionNoArg,		(char *) "off"},
{"-?",		".help",	XrmoptionNoArg,		(char *) "on"},
{"-a",		".absolute",	XrmoptionNoArg,		(char *) "on"},
{"-background",	"*background",	XrmoptionSepArg,	(char *) NULL},
{"-b",		".gray",	XrmoptionNoArg,		(char *) "off"},
{"-bg",		"*background",	XrmoptionSepArg,	(char *) NULL},
{"-bordercolor","*borderColor",	XrmoptionSepArg,	(char *) NULL},
{"-borderwidth",".borderWidth",	XrmoptionSepArg,	(char *) NULL},
{"-detailgeometry", "*detailWin.geometry", XrmoptionSepArg,	(char *) NULL},
{"-display",	".display",     XrmoptionSepArg,	(char *) NULL},
{"-f",		".used",	XrmoptionNoArg,		(char *) "off"},
{"-fg",		"*foreground",	XrmoptionSepArg,	(char *) NULL},
{"-font",	"*font",	XrmoptionSepArg,	(char *) NULL},
{"-foreground",	"*foreground",	XrmoptionSepArg,	(char *) NULL},
{"-fsb",	".fs_border",	XrmoptionNoArg,		(char *) "off"},
{"-geometry",	"*mainWin.geometry",	XrmoptionSepArg,(char *) NULL},
{"-help",	".help",	XrmoptionNoArg,		(char *) "on"},
{"-i",		".updInterval",	XrmoptionSepArg,	(char *) NULL},
{"-iconic",	".iconic",	XrmoptionNoArg,		(char *) "on"},
{"-m",		".minimize",	XrmoptionNoArg,		(char *) "1"},
{"-mb",		".menuborder",	XrmoptionNoArg,		(char *) "on"},
{"-menu",	".show_menu", 	XrmoptionNoArg,		(char *) "on"},
{"-name",	".name",	XrmoptionSepArg,	(char *) NULL},
{"-p",		".percent",	XrmoptionNoArg,		(char *) "off"},
{"-rs",		".reserved_space",XrmoptionSepArg,	(char *) "NULL"},
{"-rv",		"*reverseVideo", XrmoptionNoArg,	(char *) "on"},
{"-synchronous","*synchronous",	XrmoptionNoArg,		(char *) "on"},
{"-v",          ".verbose",     XrmoptionNoArg,         (char *) "on"},
{"-w",          ".warnval",     XrmoptionSepArg,        (char *) NULL},
{"-xrm",	NULL,		XrmoptionResArg,	(char *) NULL},
{"help",	".help",	XrmoptionNoArg,		(char *) "on"},
};


void main (argc, argv)
int argc;
char **argv;
{
	int 		i=0;
	XrmDatabase 	commandlineDB=NULL;
	char* 		myDisplayName = NULL;
	XrmString 	str_type;
	XrmValue 	value;

	/* *** Try and work out the name the application will run under *** */
	/* *** This needed for finding the correct resources. *** */
	{
	char** ptr = &argv[1];

	for (; *ptr != NULL; ptr++)
	if (strcmp(*ptr, "-name") == 0 && *(ptr + 1) != NULL) 
		{
		myname = *(ptr + 1);
		break;
		}

	if (*ptr == NULL) 
		{
		char* ptr1 = rindex(argv[0], '/');
		if (ptr1)
			myname = ++ptr1;
		else
			myname = argv[0];
		}
	}

	XrmInitialize();
	XrmParseCommand(&commandlineDB, opTable, 
		sizeof(opTable) / sizeof(opTable[0]), myname, &argc, argv);
	if (XrmGetResource(commandlineDB, 
		catlist(myname, ".name", (char*) NULL), "Xmem.Name", 
		&str_type, &value))
			myname = value.addr;
	if (XrmGetResource(commandlineDB, 
		catlist(myname, ".display", (char*) NULL), "Xmem.Display", 
		&str_type, &value))
			myDisplayName = value.addr;

	/* *** connect to X Server and assign screen *** */
	if ((mydisplay = XOpenDisplay(myDisplayName)) == NULL) 
		{
		fprintf (stderr, "Error Opening Display ... Exiting\n");
		exit (1);
		}

	/* *** process arguments and the resource database *** */
	process_databases (argc, argv, commandlineDB);

	/* *** Turn on X server synchronization if the user wants it. *** */
	XSynchronize(mydisplay, getBoolResource(resourceDB,
		catlist(myname, ".synchronous", (char*) NULL),
		"Xmem.Synchronous", False));

	/* *** get hostname *** */
	if (!(gethostname (hname, STRLENGTH))) 
		{
		if (verbose)
			printf ("Hostname = %s\n", hname);
		} 
	else
		fprintf (stderr, "Can't read hostname ...\n");

	/* *** get the file system statistics *** */
	/* *** this also handles getting the  *** */
	/* *** info from mount                *** */
	get_meminfo();
	if (verbose) 
		{
		printf ("Statistics gotten for Real & Virtual memory ...\n");
		printf("Mem:  %5dK av, %5dK used, %5dK free, %5dK shrd, %5dK buff\nUseage=%f%%\n",
			main_mem / 1024, used_mem / 1024, free_mem / 1024, 
			shared_mem / 1024, buf_mem / 1024, mem_perc[REAL]*100);
		printf("Swap: %5dK av, %5dK used, %5dK free%\nUsage=%f%%\n",
			swap_mem / 1024, used_swap / 1024, free_swap / 1024, 
			mem_perc[VIRTUAL]*100);
		}

	init_all_windows();

	/* *** Check to see of the user wants the colors reversed. *** */
	if (getBoolResource(resourceDB, 
		catlist(myname, ".reverseVideo", (char*) NULL),
		"Xmem.ReverseVideo", False)) 
		{
		long temp;
		temp = fg; 
		fg = bg ; 
		bg = temp;
		}

	/* *** create Main Program Window and set up the properties. *** */
	{
	XClassHint clshint;
	XWMHints wmhints;

	mainwin_hint.min_width = MIN_WIN_X;
	if (show_menu)
		mainwin_hint.min_height = ((MIN_INTERVAL(main_win) * 
			MAXMEMZONES) + BEGIN_NMW(main_win));
	else
		mainwin_hint.min_height = ((MIN_INTERVAL(main_win) * 
			MAXMEMZONES) + MENU_HEIGHT(main_win));
	/* mainwin_hint.min_height = ((MIN_INTERVAL(main_win) * NMW) +
		BEGIN_NMW(main_win));*/
	mainwin_hint.width = main_win.width;
	mainwin_hint.height = main_win.height;
	mainwin_hint.x = main_win.y;
	mainwin_hint.y = main_win.y;
	mainwin_hint.flags = main_win.flags | PMinSize | PSize;

	clshint.res_name  = myname;
	clshint.res_class = CLASS_NAME;

	wmhints.input = True;
	wmhints.initial_state = (getBoolResource(resourceDB,
		catlist(myname, ".iconic", (char*) NULL), 
		"Xmem.Iconic", False))
      		? IconicState : NormalState;
	wmhints.window_group = main_win.win;
	wmhints.flags = InputHint | StateHint | WindowGroupHint;

	create_window (DefaultRootWindow(mydisplay), &main_win, fg, bg);
	XSetStandardProperties (mydisplay, main_win.win, main_win.text,
		main_win.text, None, argv, argc, &mainwin_hint);
	XSetClassHint(mydisplay, main_win.win, &clshint);
	XSetWMHints(mydisplay, main_win.win, &wmhints);
	}

	/* ** Create the pixmap of the gray background. ** */
	gray_gc_val.stipple = XCreatePixmapFromBitmapData(mydisplay,
		DefaultRootWindow(mydisplay), gray1_bits, gray1_width, 
		gray1_height, fg, bg, 1);

	if (!gray_gc_val.stipple) 
		{
		fprintf (stderr, 
		   "Couldn't create gray tile ... escaping to black fill.\n");
		gray = FALSE;
		} 
	else
		gray_gc_val.fill_style = FillOpaqueStippled;

	/* *** create all the buttons using child windows *** */
	if (show_menu)
		for (i = 0; i < MENU_ITEMS ; ++i) 
			create_window (main_win.win, &menu[i], fg, bg); 
	for (i = 0; i < MAXMEMZONES; i++) 
		{
		fix_mem_win_width(i); 
		create_window (main_win.win, &mem_win[i],
			getColorResource(resourceDB, 
			catlist(myname, ".", MemType[i],
			".foreground", (char*)NULL),
			"Xmem.FileSystem.Foreground", fg),
			getColorResource(resourceDB, 
			catlist(myname, ".", MemType[i],
			".background", (char*)NULL), 
			"Xmem.FileSystem.Background", bg));

	if (gray)
		XChangeGC (mydisplay, mem_win[i].gc, GCFillStyle | GCStipple,
		&gray_gc_val);
		}

	/* *** Map window definitions onto screen *** */
	XMapRaised (mydisplay, main_win.win);
	XMapSubwindows (mydisplay, main_win.win);

	/* *** create warning pixmap *** */
	if (!(warn_pix = XCreatePixmapFromBitmapData (mydisplay, main_win.win,
		warn_bits, warn_width, warn_height, fg, bg, 
		DefaultDepth (mydisplay, DefaultScreen (mydisplay))))) 
		{
		fprintf (stderr,
			"Couldn't create Warning pixmap ... \
ignoring specified threshold.\n");
		warn_val = FALSE;
		}

#ifdef DEBUG
	fprintf (stdout, "Setup is Done!\n");
#endif 

/* ************************************************************ */
/* ************** setup is done - main event loop ************* */
/* ************************************************************ */

	do_event_loop (argc, argv);

	/* *** we are done - destroy all the windows *** */
	if (detail_open != NOGOOD) 
		{
		XFreeGC (mydisplay, global_win.gc);
		XDestroyWindow (mydisplay, global_win.win);
		}

	if (show_menu)
		destroy_menu (menu, MENU_ITEMS);
	destroy_menu (mem_win, MAXMEMZONES);
	XFreeGC (mydisplay, main_win.gc);
	XDestroyWindow (mydisplay, main_win.win);
	XCloseDisplay (mydisplay);
	exit (0);
}



/* *** This is the main event loop. Here we handle events and deal *** */
/* ***                 with all the user inputs we get             *** */
void do_event_loop (argc, argv)
int argc;
char **argv;
{
	XEvent	tevent;
	int	count, cont, this_item, i, ppnfs, done=FALSE; 
	long	target = time(NULL) + upd_interval;
	char	text[10];

    while (!done)	
	{
	/* *** here we wait while there are no events *** */
	/* *** and count out down our interval.       *** */
	while (XPending (mydisplay) == 0) 
		{
		struct timeval  sleept;
		fd_set reads;

		FD_ZERO(&reads);
		FD_SET(ConnectionNumber(mydisplay), &reads);
		sleept.tv_sec = target - time(NULL);
		sleept.tv_usec = 0; 
#ifdef HPUX
		select(ConnectionNumber(mydisplay) + 1, (int *) &reads, NULL, 
			NULL, &sleept);
#else
		select(ConnectionNumber(mydisplay) + 1, &reads, NULL, NULL, 
			&sleept);
#endif

		if ((time(NULL)) >= target) 
			{
			if (show_menu)
				ppnfs = ((main_win.height-BEGIN_NMW(main_win)) / NMW);
			else
				ppnfs = (main_win.height-MENU_HEIGHT(main_win)) / (NMW*2);
			get_meminfo ();
			main_win.height = (BEGIN_NMW(main_win) + (ppnfs * NMW));
			if (detail_open != NOGOOD)
				write_detail (detail_open);
			target = target + upd_interval;
			}
		}

	/* *** read the next event *** */
	XNextEvent (mydisplay, &myevent);

	switch (myevent.type) 
	{
	/* *** expose event -> redraw the window *** */
	 case Expose:
	/* ********************************************* *
	 *  since a window can generate multiple expose  *
	 *  events we can collapse these into one redraw *
	 * ********************************************* */
		count=0;
		do 
			{
			if (XPending (mydisplay) == 0)
				break;
			XPeekEvent (mydisplay, &tevent);
			if (tevent.xexpose.window ==  myevent.xexpose.window) 
				{
				XNextEvent (mydisplay, &myevent);
				count++;
				}
			} 
		while(tevent.xexpose.window==myevent.xexpose.window);

#ifdef DEBUG
		if (!count)
			fprintf(stdout, "got expose event in window: %d\n",
			myevent.xexpose.window);
		else
			fprintf(stdout, 
			"Compressed %d expose events into 1 in window: %d\n",
			count, myevent.xexpose.window);
#endif 

		/* ******************************************* *
		 * ** now deal with the actual expose event ** *
		 * ******************************************* */
		if (myevent.xexpose.window == main_win.win &&
		    myevent.xexpose.count == 0) 
			{
			redraw_main_win ();
			break;
			} 
		else
		if (myevent.xexpose.window == global_win.win)
			write_detail (detail_open);

		cont = TRUE;
		cont = expose_win (menu, MENU_ITEMS);
		if (!cont) break;
			for (i = 0; i < MAXMEMZONES; i++)
		if (myevent.xexpose.window == mem_win[i].win) 
			{
			redraw_mem_win (i);
			i = MAXMEMZONES;
			}
		break;

	/* *** change work window if main window changes *** */
	case ConfigureNotify:
#ifdef DEBUG
		fprintf(stdout, "got configure notify event in window: %d\n",
			myevent.xconfigure.window);
#endif
		/* *** we don't allow changes in the detail window *** */
		if (myevent.xconfigure.window != main_win.win)
			break;

		/* *** size stays the same -> nothing to do *** */
		if (myevent.xconfigure.width == main_win.width &&
		    myevent.xconfigure.height == main_win.height)
			break;

		/* *** figure out how many points per nfs we have *** */
		if (show_menu)
			ppnfs=((myevent.xconfigure.height - 
				BEGIN_NMW(main_win)) / NMW);
		else
			ppnfs=(myevent.xconfigure.height-MENU_HEIGHT(main_win))/NMW;
#ifdef DEBUG
	fprintf (stdout,
	       "got resize event - new interval = %d\nwidth = %d height = %d\n",
	       ppnfs, myevent.xconfigure.width, myevent.xconfigure.height);
#endif

		XClearWindow (mydisplay, main_win.win);
		main_win.width = myevent.xconfigure.width;
		main_win.height = myevent.xconfigure.height;
		XResizeWindow (mydisplay, main_win.win, main_win.width, 
			main_win.height);
		redraw_main_win ();
		fix_menu_pos (myevent.xconfigure.width);

		for (i = 0; i < MAXMEMZONES; i++) 
			{
			set_mem_win_size (i, ppnfs);
			XClearWindow (mydisplay, mem_win[i].win);
			XMoveResizeWindow (mydisplay, mem_win[i].win, OFF_X, 
				mem_win[i].y, mem_win[i].width, mem_win[i].height);
			/* XClearWindow (mydisplay, main_win.win); */
			if (mem_perc[i] != NOGOOD)
				redraw_mem_win (i);
			} 

		XClearWindow (mydisplay, main_win.win);
		break;

	/* *** process keyboard mapping changes *** */
	case MappingNotify:
#ifdef DEBUG
	fprintf(stdout, "got mapping notify event in window: %d\n",
		myevent.xexpose.window);
#endif

		XRefreshKeyboardMapping ((XMappingEvent *) &myevent );
		break;

      /* *** drag in work window *** */
	case MotionNotify:
#ifdef DEBUG
	fprintf(stdout, "got motion notify event in window: %d\n",
		myevent.xbutton.window);
#endif
	break;

	/* *** mouse enters a window *** */
	case EnterNotify:
		if (show_menu)
			this_item = highlight_menu (menu, MENU_ITEMS, TRUE);
		break;

	/* *** Mouse Leaves a window *** */
	case LeaveNotify:
		if (show_menu)
			this_item = highlight_menu (menu, MENU_ITEMS, FALSE);
		break;

	/* *** process mouse-button presses *** */
	case ButtonPress:
#ifdef DEBUG
	fprintf(stdout, "button press (%d) in window: %d\n",
		myevent.xbutton.button, myevent.xbutton.window);
#endif

	if (myevent.xbutton.button == 1 || myevent.xbutton.button == 2) 
		{
		this_item = which_button_pressed (menu, MENU_ITEMS);

		if (this_item == UPDATE) 
			{
			if (show_menu)
				ppnfs = ((main_win.height - BEGIN_NMW(main_win)) / NMW);
			else
				ppnfs = (main_win.height ) / (NMW*2);
				/* ppnfs = (main_win.height - MENU_HEIGHT(main_win)) / NMW; */
			/* ppnfs = ((main_win.height - BEGIN_NMW(main_win)) / NMW); */
			get_meminfo ();
			main_win.height = (BEGIN_NMW(main_win) + (ppnfs * NMW));
			if (detail_open != NOGOOD)
				write_detail (detail_open);
			redraw_main_win ();
			target = time(NULL) + upd_interval;
			break;
			} 
		else if (this_item == QUIT) 
			{
			done = TRUE;
			break;
			}

	/* *** see if btnpress was in detail window *** */
	if (myevent.xbutton.window == global_win.win) 
		{
		XFreeGC (mydisplay, global_win.gc);
		XDestroyWindow (mydisplay, global_win.win);
		detail_open = NOGOOD;
		break;
		}

	/* *** see if btnpress was in a graph window *** */
		this_item = which_button_pressed (mem_win, MAXMEMZONES);
		if (this_item == NOGOOD)
			break;

	/* *** open a detail window *** */
	if (detail_open == NOGOOD) 
		{
		open_detail_win (argc, argv);
		detail_open = this_item;
		if (detail_share_color) 
			{
			XSetBackground (mydisplay, global_win.gc, 
				mem_win[this_item].bg);
			XSetForeground (mydisplay, global_win.gc, 
				mem_win[this_item].fg);
			XSetWindowBackground(mydisplay, global_win.win, 
				mem_win[this_item].bg);
			}
		}

	/* *** det_win for this graph open - close it *** */
	else 
	if (detail_open == this_item) 
		{
		XFreeGC (mydisplay, global_win.gc);
		XDestroyWindow (mydisplay, global_win.win);
		detail_open = NOGOOD;
		} 
	else 
		{
		if (detail_share_color) 
			{
			XSetBackground (mydisplay, global_win.gc, 
				mem_win[this_item].fg);
			XSetWindowBackground(mydisplay, global_win.win, 
				mem_win[this_item].bg);
			}
		detail_open = this_item;
		write_detail (this_item);
		}
	}
	/* *** Button 3 pressed *** */
	else
		toggle_mode();
	break;

	/* *** process mouse-button release *** */
	case ButtonRelease:
#ifdef DEBUG
		fprintf(stdout, "button release in window: %d\n", 
			myevent.xbutton.window);
#endif
	break;

	/* *** process Resize *** */
	case ResizeRequest:
#ifdef DEBUG
		fprintf (stdout, "got resize event \n");
#endif
	break;

	/* *** process keyboard input *** */
	case KeyPress:
#ifdef DEBUG
		fprintf (stdout, "got keypress event in window: %d\n", 
			myevent.xkey.window);
#endif
		i = XLookupString ((XKeyEvent *)&myevent, text, 10, NULL, 0 );

		if (text[0] == UPDATE_KEY) 
			{
			if (show_menu)
				ppnfs = ((main_win.height - BEGIN_NMW(main_win)) / NMW);
			else
				ppnfs = (main_win.height) / (NMW*2);
			/* ppnfs = ((main_win.height - BEGIN_NMW(main_win)) / NMW); */
			get_meminfo ();
			main_win.height = (BEGIN_NMW(main_win) + (ppnfs * NMW));
			if (detail_open != NOGOOD)
				write_detail (detail_open);
			redraw_main_win ();
			target = time(NULL) + upd_interval;
			break;
			}
		else 
		if (text[0] == QUIT_KEY) 
			{
			done = TRUE;
			break;
			}
		else 
		if (text[0] == CLOSE_KEY) 
			{
			if (detail_open != NOGOOD) 
				{
				XFreeGC (mydisplay, global_win.gc);
				XDestroyWindow (mydisplay, global_win.win);
				detail_open = NOGOOD;
				break;
				}
			} 
		else 
		if (text[0] == TOGGLE_KEY) 
			{
			toggle_mode ();
			break;
			}
		break;
	} /* switch (myevent.type) */
    } /* while (done == 0) */
}
