/* ************************************************************************* *
   xmem - (C) Copyright 1996-1997 Robert Gasch (Robert_Gasch@peoplesoft.com)
	  http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


/* ****** include files ****** */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xresource.h>
#ifndef DYNIX
# include <stdlib.h>
#endif
#ifdef __STDC__
# include <stdarg.h>
#else
# include <varargs.h>
#endif
#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>


/* ******************************************************************** */
/* ******************** general program constants ********************* */
/* ******************************************************************** */
#define TRUE		1
#define FALSE		0
#define	STRLENGTH	255		/* length of a string */
#define NOGOOD		-1		/* to distinguish nogood from false */
#define MAXMEMZONES	2		/* real, virutal */
#define QUIT_KEY	'q'		/* quit key */
#define UPDATE_KEY	'u'		/* update key */
#define TOGGLE_KEY	't'		/* toggle mode key */
#define CLOSE_KEY	'c'		/* close detailed window key */
#define KB		1024		/* 1Kb = 1024 Bytes */
#define MB		1048576		/* 1Mb = 1048576 Bytes */
#define GB		1073741824	/* 1Gb = 1073741824 Bytes */
#define MINIMIZE	1		/* minimize */
#define CLASS_NAME	"Xmem"		/* How do we look for resources */
#define DEFAULT_FONT	"6x13"		/* default font */
#define RES_SPACE_DFLT	10		/* reserved space; original value: 10 */
#define REAL		0
#define	VIRTUAL		1
#define PROGNAME	"xmem"
#define DEF_UPD_INT	5


#ifndef APP_DEFAULTS_DIR
#define APP_DEFAULTS_DIR        "/usr/X11R5/lib/X11/app-defaults/"
#endif

					/* If something is not available we */
					/* use this to copy NA into the sting */
#define CHECKNA		if (! strcmp (s, "-1")) strcpy (s, "N.A.")
#define PRINT_KNOWN_BUGS printf ("\n")	/* at this point there are none */

					/* well, I did write the damn thing */
#define PRINT_COPYRIGHT		printf ("\
xmem v1.02 - (C) Copyright 1996-1997 Robert Gasch (Robert_Gasch@peoplesoft.com)\n\
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm\n")


/* ****** define window constants - these are positions and sizes ****** */
#define DEF_BORDER_WIDTH 	4 /* Default border width. */
#define WIN_X 			150
#define WIN_Y 			150
#define MIN_WIN_X		100
#define MENU_HEIGHT(w) 		(LETTER_HEIGHT(w))
#define MIN_MENU_HEIGHT		4
#define LETTER_HEIGHT(w)  	((w).font_info->ascent + (w).font_info->descent)
#define LETTER_ASCENT(w)  	((w).font_info->ascent)
#define LETTER_SPACE(w)	  	(LETTER_HEIGHT(w) + 5 )
#define FREE_LETTER_SP(w) 	(LETTER_SPACE(w)-LETTER_HEIGHT(w))
#define NMW_TEXT_Y 		(mem_win[i].y-(LETTER_SPACE(mem_win[i])-\
				(LETTER_HEIGHT(mem_win[i])+1)))
#define SM_MENU_WIDTH(w) 	(XTextWidth((w).font_info, (w).text, \
				strlen((w).text)))
#define MENU_ITEMS		2
#define MENU_SPACE(w)		(MENU_HEIGHT(w)*(MENU_ITEMS+2))
#define BEGIN_NMW(w)		MENU_SPACE(w)
#define MENU_Y			10
#define OFF_X			10
#define OFF_Y			10
#define INTERVAL(w)		((MENU_HEIGHT(w)+FREE_LETTER_SP(w))*2) 
#define	MIN_INTERVAL(w)		(MIN_MENU_HEIGHT+FREE_LETTER_SP(w)+\
				MENU_HEIGHT(w))
#define DETAIL_LETTER_X		13
#define DETAIL_LETTER_STRING	"XXXXXXXXXXXX "
#define DETAIL_NUM		6
#define DETAIL_X		(OFF_X*2 + XTextWidth(global_win.font_info,\
 				"Blocks Available    WWWWWWWWWWW",  \
				sizeof("Blocks Available    WWWWWWWWWWW")))
#define DETAIL_Y		((MENU_HEIGHT(global_win)*(DETAIL_NUM+3))+\
				(OFF_X*2))
/* ****** define constants used to identify menus and windows(buttons) ****** */
#define UPDATE		0
#define	QUIT		1
/* ******************** define the warning bitmap size ********************** */
#define warn_width 4
#define warn_height 10



/* ******************************************************************** */
/* ********************* program data structures ********************** */
/* ******************************************************************** */

/* *** string to hold file system name *** */
typedef char MyString[STRLENGTH];

/* ****** XWindow struct - this simplifies function calls ****** */
typedef struct {
	Window 		win;		/* window ID */
	GC		gc;		/* window graphics content */
	XFontStruct*	font_info;	/* Font the window is using. */
	char		text[40];	/* title (for menus) */
	int		x, y, 		/* position */
			width, height, 	/* size */
			line_thick;	/* line thickness of window border */
	unsigned long	fg, bg;		/* foreground and background */
	long		event_mask, 	/* which events will be registered */ 
			flags;		/* window flags */
		} WinType, *WinTypePtr;


/* ******************************************************************** */
/* ********************** function declarations *********************** */
/* ******************************************************************** */

void	do_event_loop(
#if NeedFunctionPrototypes
		      int argc,
		      char** argv
#endif
);
void	open_detail_win(
#if NeedFunctionPrototypes
		      int argc,
		      char** argv
#endif
);
void	fix_menu_pos(
#if NeedFunctionPrototypes
		     int width
#endif
);
void	fix_mem_win_width(
#if NeedFunctionPrototypes
			 int i
#endif
);
void	set_mem_win_size(
#if NeedFunctionPrototypes
			int i,
			int ppnfs
#endif
);
void get_meminfo(
#if NeedFunctionPrototypes
	       void
#endif
);
void	toggle_mode(
#if NeedFunctionPrototypes
		    void
#endif
);
void	redraw_main_win(
#if NeedFunctionPrototypes
			void
#endif
);
void 	write_detail(
#if NeedFunctionPrototypes
		     int i
#endif
);
void	write_available(
#if NeedFunctionPrototypes
		        int i
#endif
);
void 	write_percent(
#if NeedFunctionPrototypes
		      int i
#endif
);
void 	redraw_mem_win(
#if NeedFunctionPrototypes
		      int i
#endif
);
void	open_global_win(
#if NeedFunctionPrototypes
		    int	argc,
		    char **argv
#endif
);
void 	create_window (
#if NeedFunctionPrototypes
		       Window parent_win,
		       WinTypePtr this_win,
		       unsigned long foreg,
		       unsigned long backg
#endif
);
int	highlight_menu(
#if NeedFunctionPrototypes
		       WinType menu[],
		       int menu_num,
		       int hightlight
#endif
);
int	expose_win(
#if NeedFunctionPrototypes
		   WinType menu[],
		   int menu_num
#endif
);
void	destroy_menu(
#if NeedFunctionPrototypes
		     WinType menu[],
		     int menu_num
#endif
);
int	which_button_pressed(
#if NeedFunctionPrototypes
			     WinType menu[],
			     int menu_num
#endif
);
void	process_databases(
#if NeedFunctionPrototypes
			  int argc,
			  char** argv,
			  XrmDatabase commandlineDB
#endif
);
void	do_help(
#if NeedFunctionPrototypes
			void
#endif
);
void 	init_all_windows(
#if NeedFunctionPrototypes
			 void
#endif
);
Bool	getBoolResource(
#if NeedFunctionPrototypes
			XrmDatabase db,
			char* str_name,
			char* str_class,
			Bool deflt
#endif
);
int	getIntResource(
#if NeedFunctionPrototypes
		       XrmDatabase db,
		       char* str_name,
		       char* str_class,
		       int deflt
#endif
);
unsigned long getColorResource(
#if NeedFunctionPrototypes
			       XrmDatabase db,
			       char* str_name,
			       char* str_class,
			       unsigned long deflt
#endif
);
XFontStruct* getFontResource(
#if NeedFunctionPrototypes
			     XrmDatabase db,
			     char* str_name,
			     char* str_class,
			     XFontStruct* deflt
#endif
);
char*	catlist(
#if NeedFunctionPrototypes
		char*,
		...
#endif
);

#if defined (DYNIX) || defined (MACH)
char*  strstr(
#if NeedFunctionPrototypes
	      const char*,
	      const char*
#endif
);
#endif

