/* ************************************************************************* *
   xmem - (C) Copyright 1996-1997 Robert Gasch (Robert_Gasch@peoplesoft.com)
	  http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */



#include "xmem.h"

extern Display			*mydisplay;
extern char*			myname;			/* Name we hunt resources with */
extern XrmDatabase		resourceDB;		/* X resources. */
extern WinType			main_win,		/* main window */
				menu[],			/* menu items */
				mem_win[],		/* FS graphs */
				global_win;		/* compatability in util.c */
extern XGCValues		gray_gc_val;		/* GC values for gray fill */
extern XEvent			myevent;		/* event */
extern unsigned long		fg, bg;			/* forgrund, background */
extern Pixmap			warn_pix;		/* Warning Pixmap */
extern int 			upd_interval,		/* when do we update stats */
				NMW,			/* number of file systems */
				detail_open,		/* which mem_wind is open */
				detail_share_color, 	/* */
				minimize,		/* minimize dimensions */
				menu_border,		/* menu border */
				fs_border,		/* file system border */
				percent,		/* show percent */
				absolute,		/* make bar size relative */
				verbose,		/* verbose mode */
				sort,			/* do we sort by name? */
				gray,			/* use gray fill */
				available,		/* do we show what's available */
				show_menu;		/* do we show the menu */

extern unsigned int 		main_mem, 
				used_mem, 
				free_mem, 
				shared_mem, 
				buf_mem,
				cached_mem,
				swap_mem, 
				used_swap, 
				free_swap,
				mem[], 
				biggest_mem;

extern unsigned long		turns;			/* how many turns */
extern float			mem_perc[],		/* percentage use for memory */
				warn_val;

extern MyString			MemType[];


static	int			old_meminfo=0;




/* ************************************************************************ */
/* ************** set the window's width in relative mode ***************** */
/* ************************************************************************ */
void fix_mem_win_width (i)
int i;
{
	if (!absolute)
		{
		mem_win[i].width=(int)(mem_win[i].width*
			((double)mem[i]/(double)biggest_mem));
		if (mem_win[i].width < 1) 
			mem_win[i].width=1;
		}
}




/* *********************************************************************** */
/* ******************** write out the detailed info ********************** */
/* *********************************************************************** */
void write_detail (i)
int i;
{
	int 	c=3, 
		lhs = XTextWidth(global_win.font_info, "Blocks Available ",
			sizeof ("Blocks Available "));
	char 	s[30]; 

	XClearWindow (mydisplay, global_win.win);
	/* *** write the detailed info *** */
	XDrawImageString (mydisplay, global_win.win, global_win.gc, 
		((global_win.width / 2) - (XTextWidth(global_win.font_info,
		MemType[i], strlen(MemType[i])) / 2)), 
		OFF_Y+MENU_HEIGHT(global_win), MemType[i], strlen(MemType[i]));

	XDrawImageString (mydisplay, global_win.win, global_win.gc, 
		OFF_X, OFF_Y+MENU_HEIGHT(global_win)*c, "Available", 
		strlen ("Available"));
	if (i==REAL)
		sprintf (s, "%6dK", main_mem/KB);
	else
		sprintf (s, "%6dK", swap_mem/KB);
	XDrawImageString (mydisplay, global_win.win, global_win.gc, 
		OFF_X+lhs, OFF_Y+MENU_HEIGHT(global_win)*c++, s, strlen(s));

	XDrawImageString (mydisplay, global_win.win, global_win.gc, 
		OFF_X, OFF_Y+MENU_HEIGHT(global_win)*c, "Used", 
		strlen ("Used"));
	if (i==REAL)
		sprintf (s, "%6dK", used_mem/KB);
	else
		sprintf (s, "%6dK", used_swap/KB);
	XDrawImageString (mydisplay, global_win.win, global_win.gc, 
		OFF_X+lhs, OFF_Y+MENU_HEIGHT(global_win)*c++, s, strlen(s));

	XDrawImageString (mydisplay, global_win.win, global_win.gc, 
		OFF_X, OFF_Y+MENU_HEIGHT(global_win)*c, "Free", 
		strlen ("Free"));
	if (i==REAL)
		sprintf (s, "%6dK", free_mem/KB);
	else
		sprintf (s, "%6dK", (swap_mem-free_swap)/KB);
	XDrawImageString (mydisplay, global_win.win, global_win.gc, 
		OFF_X+lhs, OFF_Y+MENU_HEIGHT(global_win)*c++, s, strlen(s));

	if (i==REAL)
		{
		XDrawImageString (mydisplay, global_win.win, global_win.gc, 
			OFF_X, OFF_Y+MENU_HEIGHT(global_win)*c, "Shared", 
			strlen ("Shared"));
		sprintf (s, "%6dK", shared_mem/KB);
		XDrawImageString (mydisplay, global_win.win, global_win.gc, 
			OFF_X+lhs, OFF_Y+MENU_HEIGHT(global_win)*c++, s, strlen(s));

		XDrawImageString (mydisplay, global_win.win, global_win.gc, 
			OFF_X, OFF_Y+MENU_HEIGHT(global_win)*c, "Buffer", 
			strlen ("Buffer"));
		sprintf (s, "%6dK", buf_mem/KB);
		XDrawImageString (mydisplay, global_win.win, global_win.gc, 
			OFF_X+lhs, OFF_Y+MENU_HEIGHT(global_win)*c++, s, strlen(s));

		if (!old_meminfo)
			{
			XDrawImageString (mydisplay, global_win.win, global_win.gc, 
				OFF_X, OFF_Y+MENU_HEIGHT(global_win)*c, 
				"Cached", strlen ("Cached"));
			sprintf (s, "%6dK", cached_mem/KB);
			XDrawImageString (mydisplay, global_win.win, global_win.gc, 
				OFF_X+lhs, OFF_Y+MENU_HEIGHT(global_win)*c++, 
				s, strlen(s));
			}
		}
}




/* *********************************************************************** */
/* ********************** get the memory usage info ********************** */
/* ******************** this routine stolen from top.c ******************* */
/* *********************************************************************** */
void get_meminfo (void)
{
	char	*titles[6], name[40], buf1[80], buf2[80];
	int	i, n, tint[6];

	close(0);
	if (open("/proc/meminfo", O_RDONLY ) < 0)
		{
		perror(PROGNAME ": Couldn't open /proc/meminfo");
		exit (1);
		}

	/* *** old_meminfo is for pre-2.0 kernels. I'm not sure where *** */
	/* *** format of /proc/meminfo changed, but I noticed the     *** */
	/* *** change after having upgraded from 1.2.13 to 2.0.18.    *** */

	/* *** get the column titles *** */
	fseek(stdin, 0L, SEEK_SET); 
	fgets(buf1, 80, stdin);
	for (i=0; i<6; i++) 
		{
		titles[i] = strtok( ( i ? NULL : buf1 ), " \t:\n" );
		if (!titles[i]) 
			{
			if (i != 5) 	/* *** sanity check *** */
				{
				fprintf( stderr, "free: error parsing column titles from /proc/meminfo. Parsed fields:\n" );
				for (i=0; i<6; i++)
					fprintf (stderr, "%s ", titles[i]);
				exit (1);
				} 
			else
				old_meminfo=TRUE;
			}
       		 }

	if (!turns)
		if (verbose)
			{
			if (old_meminfo)
				printf ("Using pre v2.0 /proc/meminfo format ...\n");
			else
				printf ("Using v2.0 /proc/meminfo format ...\n");
			}

	/* *** read and translate data lines *** */
	for (i=0; i<2;i++)
		{
		fgets(buf2, 80, stdin);
		if (old_meminfo) 
			{
			n = sscanf(buf2, "%s %d %d %d %d %d", name,
				&tint[0], &tint[1], &tint[2], &tint[3], 
				&tint[4]);
			} 
		else 
			{
			n = sscanf(buf2, "%s %d %d %d %d %d %d", name,
				&tint[0], &tint[1], &tint[2], &tint[3], 
				&tint[4], &tint[5]);
			}

		/* *** sanity check *** */
		if (n < 4)
			{
			fprintf( stderr, "Error Parsing Data Lines: Name='%s'.\n", name );
			exit (1);
			}

		/* *** assign proper values *** */
		if (strstr (name, "Mem"))
			{
			main_mem=tint[0];
			used_mem=tint[1];
			free_mem=tint[2];
			shared_mem=tint[3];
			buf_mem=tint[4];
			if (!old_meminfo)
				cached_mem=tint[5];
			}
		else
		if (strstr (name, "Swap"))
			{
			swap_mem=tint[0];
			used_swap=tint[1];
			free_swap=tint[1];
			}
		else
			{
			fprintf( stderr, "Error Parsing Mem && Swap Lines: Name='%s'.\n", name );
			exit (1);
			}
		}

	mem[0]=main_mem;
	mem[1]=swap_mem;
	mem_perc[0]=(1-((double)free_mem+(double)buf_mem)/(double)main_mem);
	if (used_swap)
		mem_perc[1]=(1-(double)used_swap/(double)swap_mem);
	else
		mem_perc[1]=0;

	if (main_mem > swap_mem)
		biggest_mem=main_mem;
	else
		biggest_mem=swap_mem;

#ifdef DEBUG
	printf("Mem:  %5dK av, %5dK used, %5dK free, %5dK shrd, %5dK buff, usage=%f%%\n",
		main_mem / 1024, used_mem / 1024, free_mem / 1024, 
		shared_mem / 1024, buf_mem / 1024, mem_perc[0]*100);
	printf("Swap: %5dK av, %5dK used, %5dK free, usage=%f%%\n",
		swap_mem / 1024, used_swap / 1024, free_swap / 1024, 
		mem_perc[1]*100);
#endif /* DEBUG */

	for (i=0; i<MAXMEMZONES; i++)
		{
		mem_win[i].width=main_win.width-OFF_X*2;
		/* *** don't do this the first time *** */
		if (turns)
			fix_mem_win_width (i);
		if (turns)
			{
			XResizeWindow (mydisplay, mem_win[i].win, 
				mem_win[i].width, mem_win[i].height);
			if (mem_perc[i]!=NOGOOD) 
				redraw_mem_win (i);
			}
		}
	turns++;
}






/* ***************************************************************** */
/* ****************** Process the program arguments **************** */
/* ***************************************************************** */
void process_databases (argc, argv, commandlineDB)
int argc;
char **argv;
XrmDatabase commandlineDB;
{
	char name[MAXPATHLEN];
	char msg[255];

	strcpy (msg, "");
	/* *** Load the local app-defaults file. *** */
	if (getenv ("XAPPLRESDIR"))
		{
		strcpy(name, getenv ("XAPPLRESDIR"));
		strcat(name, "/");
		strcat(name, CLASS_NAME);
		resourceDB = XrmGetFileDatabase(name);
		if (resourceDB)
			sprintf (msg, 
				"Read Resource definitions in %s ...", name);
		}
		

	/* *** check if we really found something *** */
	/* *** if not, check in APP_DEFAULTS_DIR *** */
	if (!resourceDB) 
		{
		strcpy(name, APP_DEFAULTS_DIR);
		strcat(name, "/");
		strcat(name, CLASS_NAME);
		resourceDB = XrmGetFileDatabase(name);
		if (resourceDB)
			sprintf(msg, "Read Resource definitions in %s ...",
				name);
		}

	/* *** Load in any definitions supplied in the X server. *** */
	if (XResourceManagerString(mydisplay) != NULL)
		XrmMergeDatabases(XrmGetStringDatabase(XResourceManagerString
			(mydisplay)), &resourceDB);

	/* *** Merge in the command line arguments. *** */
	XrmMergeDatabases(commandlineDB, &resourceDB);

	/* *** See if the help flag was set. *** */
	if (getBoolResource(resourceDB, catlist(myname, ".help", (char*)NULL),
		"Xmem.Help", False))
			do_help();

	/* *** See if the verbose flag was set. *** */
	if ((verbose = getBoolResource(resourceDB,
		catlist(myname, ".verbose", (char*) NULL), 
		"Xmem.Verbose", False))) 
		{
		PRINT_COPYRIGHT;
		puts ("-v flag caught - verbose mode enabled...");
		if (resourceDB && strlen (msg))
			puts (msg);
		}

	/* *** this one has to go befor the -fg evaluation   *** */
	/* *** as otherwise the default supplied to the      *** */
	/* *** evaluation function will override our setting *** */
	if (!(gray = getBoolResource(resourceDB,
		catlist(myname, ".gray", (char*) NULL), "Xmem.Gray", True)))
		if (verbose) 
			puts("-b flag caught - will draw bars black.");

	/* *** assign foreground and backgound colors *** */
 	fg = getColorResource(resourceDB, catlist(myname, 
		".foreground", (char*) NULL), "Xmem.Foreground",
		BlackPixel (mydisplay, DefaultScreen (mydisplay))); 
	if (fg != BlackPixel (mydisplay, DefaultScreen (mydisplay)))
		{
		if (verbose)
			printf ("-fg flag caught - foreground set and \
gray fill disabled.\n");
		gray=FALSE;
		}
	bg = getColorResource(resourceDB, catlist(myname, 
		".background", (char*) NULL), "Xmem.Background",
		WhitePixel (mydisplay, DefaultScreen (mydisplay)));
	if (bg != WhitePixel (mydisplay, DefaultScreen (mydisplay)))
		if (verbose) 
			printf ("-bg flag caught - background set\n");

	if ((minimize = getIntResource(resourceDB,
		catlist(myname, ".minimize", (char*) NULL), 
		"Xmem.Minimize", False)))
		if (verbose)
			puts("-m flag caught - will minimize \
window size.");

	if ((menu_border = getBoolResource(resourceDB,
		catlist(myname, ".menuborder", (char*)NULL),
		"Xmem.Menuborder", False)))
		if (verbose)
			puts("-mb flag caught - will draw menu border.");

	if ((fs_border = getBoolResource(resourceDB,
		catlist(myname, ".fs_border", (char*)NULL),
		"Xmem.Fs_border", True)));
	else
		if (verbose)
			puts("-fsb flag caught - will not draw file system \
border.");

	if (!(percent = getBoolResource(resourceDB,
		catlist(myname, ".percent", (char*) NULL),
		"Xmem.Percent", True)))
		{
		available=FALSE;
		if (verbose) 
			puts("-p flag caught - will turn percent display off.");
		}

	if ((show_menu = getBoolResource(resourceDB,
		catlist(myname, ".show_menu", (char*) False),
		"Xmem.Show_menu", False)))
		{
		show_menu=TRUE;
		if (verbose) 
			puts("-menu flag caught - will show menu.");
		}

	if ((absolute = getBoolResource(resourceDB,
		catlist(myname, ".absolute", (char*) NULL), 
		"Xmem.Absolute", False)))
		if (verbose)
			puts("-a flag caught - will display sizes relative \
to absolute FS size.");

	if ((detail_share_color = getBoolResource(resourceDB,
		catlist(myname, ".detailWin.share", (char*) NULL),
		"Xmem.detailWin.Share", False)))
		if (verbose)
		puts ("Detail window will have the same color as the \
file system it is displaying.");

	if ((warn_val = (float) getIntResource(resourceDB,
		catlist(myname, ".warnval", (char*)NULL),
		"Xmem.Warnval", 0)) != 0.0) 
		{
		if (warn_val < 1.0 || warn_val > 99.0) 
			{
			puts("Illegal value supplied for warning threshold ... \
resetting.");
			warn_val = 0.0;
			} 
		else
			{
			if (verbose)
				printf ("-w flag caught - setting warning \
threshold to %f %%.\n", warn_val);
			warn_val /= 100.0;
			}
		}

	if ((upd_interval = getIntResource(resourceDB,
		catlist(myname, ".updInterval", (char*)NULL),
		"Xmem.UpdInterval", DEF_UPD_INT)) != DEF_UPD_INT) 
		{
		if (upd_interval < 1) 
			{
			fprintf (stderr, "Supplied interval (%d) too small \
(minimum = 1).\n", upd_interval);
			exit (1);
			}
		if (verbose)
			printf ("-i flag caught - update interval changed \
to %d sec.\n", upd_interval);
		}
}


void do_help ()
{
	printf ("\n\
                        X   X  M     M  EEEEEE  M     M \n\
                         X X   MM   MM  E       MM   MM \n\
                          X    M M M M  EEEEEE  M M M M \n\
                         X X   M  M  M  E       M  M  M \n\
                        X   X  M     M  EEEEEE  M     M \n");
	printf ("\n\
displays bar graphs for the real and virtual memory usage on your machine.\n\
Clicking on a bar graph gives you detailed information - clicking in the\n\
detail window or clicking in it again closes the detail window. Use %c or the\n\
right mouse button to toggle between absolute and relative display modes.\n\
The following options are supported:\n", TOGGLE_KEY);

printf ("\n\
        +rv                     reverse video (use to override xrdb entry)\n\
        +synchronous            syncronous mode (use to override xrdb entry)\n\
        -?                      help\n\
        -a                      absolute display mode\n\
        -background <arg>       backgound color \n\
        -b                      black bars (disable gray fill)\n\
        -bg <arg>               same as -background\n\
        -bordercolor <arg>      border color\n\
	-detailgeometry         Geometry of detail window. Only X and Y are\n\
					honored\n\
	-display		display\n\
	-fg			same as -foreground\n\
        -font <arg>             font\n\
        -foreground <arg>       forground color (also file system bar color)\n\
	-fsb			draw bars without border\n\
        -geometry <arg>         geometry (will override extreme window sizes\n\
                                        to apply reasonable settings)\n\
        -help                   help\n\
        -i <arg>                interval at which stats are updated\n\
					default = 60s\n\
	-iconic                 iconic\n\
        -m                      minimize window size\n\
        -mb                     draw menu borders\n\
        -name <arg>             run xmem under this name\n\
	-p                      don't display percentages\n\
        -rv                     reverse video\n\
        -synchronous            synchronous mode\n\
        -v                      verbose\n\
        -w <arg>                display warning pixmap when (real) memory\n\
					usage reaches <arg>%\n\
        -xrm                    set an entry in the resource database for\n\
                                        this execution only\n\
        help                    get this description\n");


	PRINT_KNOWN_BUGS;
	PRINT_COPYRIGHT;
	exit (0);
}

